package apps;

import java.awt.Desktop;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileFilter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLClassLoader;

import com.ibm.rhapsody.apps.utils.Reporter;

import docgen.SampleDocgen;

public class MBDGDocumentGenerator {

	private MBDGConfiguration configuration;
	
	private String libs = "";
	private String docgenBatchPath = "";
	private SampleDocgen sampleDocgen;

	public MBDGDocumentGenerator(MBDGConfiguration configuration) {
		this.configuration = configuration;
		
		setLibs(configuration.getDocgenPath()+"\\lib");
		setDocgenBatchPath(configuration.getDocgenPath()+"\\docgen.bat");
		
		loadRPELibraries();		
		
		//SampleDocgen.configureLogging(configuration.getRPEProductHome());
		sampleDocgen = new SampleDocgen();
	}

	public void generateAndOpen() {
		new Thread(new Runnable() {
			@Override
			public void run() {
				try {
					generate();
					open();
				} finally {
					MBDGUtils.enableNewTermSupportInFullForMBDG(false);					
				}
			}
		}).start();
	}

	public void generate() {
		try {
			//startDocgen();
			genarateReport();
           	Reporter.report("\tDocument Path: " + configuration.getDocumentPath());
			
		} catch (Exception e) {
			Reporter.report(e);
		}
	}
	
	protected void genarateReport() {
		sampleDocgen.initialize(configuration.getDocumentSpecificationPath(), configuration.getRPEProductHome(), "", false);
		sampleDocgen.generateReport(configuration.getDocumentPath());
	}
	
	protected void startDocgen() throws IOException {
		Process process = new ProcessBuilder(getExecuteCommand()).start();
		
		InputStream is = process.getInputStream();
		InputStreamReader isr = new InputStreamReader(is);
		BufferedReader br = new BufferedReader(isr);
		String line;

		while ((line = br.readLine()) != null) {
		  Reporter.report(line);
		}		
	}
	
	protected String getExecuteCommand() {
		String java = System.getProperty("java.home")+"\\bin\\java";
		String command = String.format("\"%s\" \"%s\" \"%s\" \"%s\" \"%b\" \"%s\"", getDocgenBatchPath(), configuration.getDocumentSpecificationPath(), configuration.getDocgenPath(), configuration.getDocumentPath(), configuration.isDebugMode(), java);
		return command;
	}
	
	public void open() {
		Reporter.report("\tOpening Document: " + configuration.getDocumentPath() + " ... ");
		try {
			Desktop desktop = null;
			if (Desktop.isDesktopSupported()) {
			    desktop = Desktop.getDesktop();
				desktop.open(new File(configuration.getDocumentPath()));
			}
		} catch (IOException e) {
			Reporter.report(e);
		}
	}

	@SuppressWarnings({ "rawtypes", "unchecked", "deprecation" })
	protected void addURL(String path) throws Exception {
		URL url = new File(path).toURL();
		URLClassLoader classLoader = (URLClassLoader) ClassLoader.getSystemClassLoader();
		Class clazz= URLClassLoader.class;
		// Use reflection
		Method method= clazz.getDeclaredMethod("addURL", new Class[] { URL.class });
		method.setAccessible(true);
		method.invoke(classLoader, new Object[] { url });
	}	
	
	protected void loadRPELibraries() {
		new File(configuration.getRPELibrariesDir()).listFiles(new FileFilter() {
			public boolean accept(File pathname) {
				if (pathname.getName().endsWith(".jar")) {
					try {
						addURL(pathname.getAbsolutePath());
					} catch (Exception e) {
						Reporter.report(e);
					}	
				}
				return true;
			}
		});
	}	

	public String getLibs() {
		return libs;
	}

	public void setLibs(String libs) {
		this.libs = libs;
	}

	public String getDocgenBatchPath() {
		return docgenBatchPath;
	}

	public void setDocgenBatchPath(String docgenBatchPath) {
		this.docgenBatchPath = docgenBatchPath;
	}
}
