package apps;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import com.telelogic.rhapsody.core.IRPClassifier;
import com.telelogic.rhapsody.core.IRPCollection;
import com.telelogic.rhapsody.core.IRPDependency;
import com.telelogic.rhapsody.core.IRPMatrixView;
import com.telelogic.rhapsody.core.IRPModelElement;
import com.telelogic.rhapsody.core.IRPStereotype;
import com.telelogic.rhapsody.core.IRPTableLayout;
import com.telelogic.rhapsody.core.IRPTableView;
import com.telelogic.rhapsody.core.IRPTag;

public class MBDGUtils {

	public static String getTagValue(IRPModelElement e, String name) {
		String value = null;
		IRPTag tag = e.getTag(name);
		if (tag != null) {
			value = tag.getValue();
		}
		return new MBDGExpander().expand(value);
	}

	public static String createTempDirectory(String dir) throws IOException {
		String tmpdir = System.getProperty("java.io.tmpdir");
		if (tmpdir == null) {
			throw new IOException("System property 'java.io.tmpdir' does not specify a temporary directory");
		}

		File tmpDir = new File(tmpdir+File.separator+dir);
		
		if (!tmpDir.exists()) {
			boolean created = tmpDir.mkdirs();
			if (!created) {
				throw new IOException("Unable to create the temporary directory: " + tmpDir);
			}
		}

		return tmpDir.getAbsolutePath();
	}
	
	public static boolean isType(MBDGModelElement modelElement, String type) {	
		return isType(modelElement.element, type);
	}
	
	public static boolean isType(MBDGElement modelElement, String type) {	
		return isType(modelElement.getElement(), type);
	}	
	
	public static boolean isType(IRPModelElement element, String type) {
		boolean res = false;
		List<IRPStereotype> stereotypes = getStereotypes(element);
		for (IRPStereotype stereotype : stereotypes) {
			String name = stereotype.getName();
			if (name.equals(type)) {
				res = true;
				break;
			}
		}
		return res;
	}
	
	public static List<IRPStereotype> getStereotypes(IRPModelElement element) {
		List<IRPStereotype> list = new ArrayList<IRPStereotype>();
		if (element != null) {
			@SuppressWarnings("unchecked")
			List<IRPStereotype> stereotypes = element.getStereotypes().toList();
			for (IRPStereotype stereotype : stereotypes) {
				list.add(stereotype); 
				getBaseStereotypes(stereotype, list);
			}
		}
		return list;
	}		
	
	public static void getBaseStereotypes(IRPStereotype stereotype, List<IRPStereotype> stereotypes) {
		@SuppressWarnings("unchecked")
		List<IRPClassifier>baseStereotypes = stereotype.getBaseClassifiers().toList();
		for (IRPClassifier baseStereotype : baseStereotypes) {
			if (baseStereotype instanceof IRPStereotype) {
				stereotypes.add((IRPStereotype)baseStereotype);
				getBaseStereotypes((IRPStereotype)baseStereotype, stereotypes);
			}
		}
	}	
	
	public static void enableNewTermSupportInFullForMBDG(boolean enable) {
		String value = String.valueOf(enable);
		IRPCollection params = MBDGExpander.getRhapsody().createNewCollection();
		params.setSize(1);
		params.setString( 1, value);
		MBDGExpander.getRhapsody().executeCommand( "NewTermSupportInFullForMBDG", params, null);
		params.empty();		
	}	
	
	public static String toString(IRPCollection col) {
		String res = "";
		List<?> elems = col.toList();
		for (Object elem : elems) {
			if (!res.equals("")) {
				res += ",";
			}
			if (elem instanceof IRPModelElement) {
				res += ((IRPModelElement)elem).getName();
			} else {
				res += elem.toString();
			}
		}
		return res;
	}	
	
	public static List<Object> getReferencedItems(IRPModelElement modelELement, MBDGModelElement baseModelElement) {
		List<Object> items = new ArrayList<Object>();
		if (modelELement instanceof IRPTableView || modelELement instanceof IRPTableLayout || modelELement instanceof IRPMatrixView) {
			IRPModelElement scope = (baseModelElement != null ? baseModelElement.element : null);
			List<Object> elements = getResultList(modelELement, scope);
			items.addAll(elements);
		} else {
			items.add(modelELement);			
		}
		return items;
	}	
	
	public static List<MBDGModelElement> getReferencedModelElementsOnly(IRPModelElement docElement, MBDGModelElement baseModelElement) throws IOException {
		List<MBDGModelElement> elements = new ArrayList<MBDGModelElement>();
		@SuppressWarnings("unchecked")
		List<IRPDependency> dependencies = docElement.getDependencies().toList();
		for (IRPDependency dependency : dependencies) {
			IRPModelElement modelElement = dependency.getDependsOn();
			if (modelElement != null) {
				boolean isScopeReference = MBDGUtils.isType(dependency, "DocScopeReference");
				boolean isElementReference = MBDGUtils.isType(dependency, "DocElementReference");
				if (isElementReference || isScopeReference) {
					List<Object> items = getReferencedItems(modelElement, baseModelElement);
					for (Object object : items) {
						if (object instanceof IRPModelElement) {
							MBDGModelElement item = new MBDGModelElement((IRPModelElement)object);
							elements.add(item);
						}
					}
				}
			}
		}
		return elements;
	}
	
	public static boolean isAttachedContentModelElement(MBDGModelElement element) throws IOException {
		MBDGModelElement scopeModelElement = null; 
		List<MBDGModelElement> elements = getContentModelElements(element, scopeModelElement);
		return (elements.size() != 0);
	}	
	
	public static List<MBDGModelElement> getScopeModelElements(MBDGModelElement element) throws IOException {
		return getModelElements(element, "DocScopeReference", null, true);
	}
	
	public static List<MBDGModelElement> getContentModelElementsExpandQueries(MBDGModelElement element, List<MBDGModelElement> scopeModelElements) throws IOException {
		return getModelElements(element, "DocElementReference", scopeModelElements, true);
	}
	
	public static List<MBDGModelElement> getContentModelElementsExpandQueries(MBDGModelElement element, MBDGModelElement scopeModelElement) throws IOException {
		List<MBDGModelElement> scopeModelElements = new ArrayList<MBDGModelElement>();
		if(scopeModelElement != null) {
			scopeModelElements.add(scopeModelElement);
		}
		return getModelElements(element, "DocElementReference", scopeModelElements, true);
	}	
	
	public static List<MBDGModelElement> getContentModelElements(MBDGModelElement element, List<MBDGModelElement> scopeModelElements) throws IOException {
		return getModelElements(element, "DocElementReference", scopeModelElements, false);
	}
	
	public static List<MBDGModelElement> getContentModelElements(MBDGModelElement element, MBDGModelElement scopeModelElement) throws IOException {
		List<MBDGModelElement> scopeModelElements = new ArrayList<MBDGModelElement>();
		if(scopeModelElement != null) {
			scopeModelElements.add(scopeModelElement);
		}		
		return getModelElements(element, "DocElementReference", scopeModelElements, false);
	}	
	
	public static List<MBDGModelElement> getModelElements(MBDGModelElement modelElement, String type, List<MBDGModelElement> scopes, boolean expandQuery) throws IOException {
		List<MBDGModelElement> elements = new ArrayList<MBDGModelElement>();
		if (modelElement == null || modelElement.element == null) {
			return elements;
		}
		@SuppressWarnings("unchecked")
		List<IRPDependency> dependencies = modelElement.element.getDependencies().toList();
		for (IRPDependency dependency : dependencies) {
			// Look for elements by dependency type
			if (MBDGUtils.isType(dependency, type)) {			
				IRPModelElement dependsOn = dependency.getDependsOn();
				// Document element can't be scope
				if (MBDGUtils.isType(dependsOn, "DocElement")) {
					continue;
				}			
				MBDGModelElement depModelElement = new MBDGModelElement(dependsOn);
				// Each element collected by from query or table is scope 
				if (dependsOn instanceof IRPTableView || dependsOn instanceof IRPTableLayout || dependsOn instanceof IRPMatrixView) {
					if (expandQuery) {
						List<MBDGModelElement> tempScopes = getScopeModelElements(depModelElement);
						// Append inherited scope elements as scope
						if (scopes != null) {
							tempScopes.addAll(scopes);
						}
						for (MBDGModelElement scope : tempScopes) {
							List<MBDGModelElement> modelElements = getReferencedModelElements(dependsOn, scope.element);
							elements.addAll(modelElements);			
						}
						if (tempScopes.size() == 0) {
							List<MBDGModelElement> modelElements = getReferencedModelElements(dependsOn, null);
							elements.addAll(modelElements);	
						}
					} else {
						elements.add(depModelElement);
					}
				} else {
					elements.add(depModelElement);
				}
			}
		}
		return elements;
	}

	public static List<MBDGModelElement> getReferencedModelElements(IRPModelElement query, IRPModelElement scope) {
		List<MBDGModelElement> modelElements = new ArrayList<MBDGModelElement>();
		if (query instanceof IRPTableView || query instanceof IRPTableLayout || query instanceof IRPMatrixView) {
			List<Object> elements = getResultList(query, scope);
			for (Object object : elements) {
				if (object instanceof IRPModelElement) {
					IRPModelElement element = (IRPModelElement)object;
					if (element.getName().equals("TopLevel")) { 
							continue;
					}
					MBDGModelElement modelElement = new MBDGModelElement(element);
					modelElements.add(modelElement);
				}
			}
		}
		return modelElements;
	}	
	
	public static List<Object> getResultList(IRPModelElement query, IRPModelElement scope) {
		IRPCollection pCommandInitialization = MBDGExpander.getRhapsody().createNewCollection();
		pCommandInitialization.addItem(query);
		if (scope != null) {
			pCommandInitialization.addItem(scope);
		}
		IRPCollection pCommandResult = MBDGExpander.getRhapsody().createNewCollection();
		String command = (query instanceof IRPMatrixView ? "GetResultMatrixFromTable" : "GetResultListFromTable");
		MBDGExpander.getRhapsody().executeCommand(command, pCommandInitialization, pCommandResult);
		@SuppressWarnings("unchecked")
		List<Object> elements= pCommandResult.toList();
		return elements;
	}
	
	public static String getKeywordTagValue(IRPModelElement e) {
		String keyword = getTagValue(e, "Keyword");
		if (keyword == null || keyword.equals("")) {
			keyword = "${MBDG_CONTENT}"; 
		}
		return keyword;
	}	
	
	public static boolean isDanglingElement(IRPModelElement docElement) throws IOException {	
		boolean res = true;
		@SuppressWarnings("unchecked")
		List<IRPDependency> dependencies = docElement.getDependencies().toList();
		for (IRPDependency dependency : dependencies) {
			// Look for elements by dependency type
			if (MBDGUtils.isType(dependency, "DocElementReference")) {			
				res = false;
				break;
			}
		}
		return res;
	}
	
	public static String getRootTemplatePath(MBDGElement root) throws IOException {
		String path = new MBDGExpander().expand(MBDGConfiguration.DocumentTemplatePath);
		validateTemplatePath(path, root);
		return path;
	}
	
	public static String getTemplatePathByTag(MBDGElement docElement) throws IOException {
		String path = MBDGUtils.getTagValue(docElement.getElement(), "templateFile");
		if (path == null) {
			throw new IOException(String.format("Element [%s] is missing 'templateFile' tag.", docElement.getElement().getName()));
		}
		validateTemplatePath(path, docElement);		
		return path;
	}
	

	public static void validateTemplatePath(String templatePath, MBDGElement docElement) throws IOException {
		if (templatePath.equals("")) {
			throw new IOException(String.format("Element [%s] has empty templateFile tag.", docElement.getElement().getName()));
		}
	}	
}
