package com.ibm.rhapsody.sysML;

import java.util.List;

import com.telelogic.rhapsody.core.IRPActivityDiagram;
import com.telelogic.rhapsody.core.IRPApplication;
import com.telelogic.rhapsody.core.IRPClassifier;
import com.telelogic.rhapsody.core.IRPCollection;
import com.telelogic.rhapsody.core.IRPFlowchart;
import com.telelogic.rhapsody.core.IRPGraphNode;
import com.telelogic.rhapsody.core.IRPInstance;
import com.telelogic.rhapsody.core.IRPModelElement;
import com.telelogic.rhapsody.core.IRPSwimlane;
import com.telelogic.rhapsody.core.RhapsodyAppServer;

public class CreateSwimlaneFromParts {
	private IRPApplication rhapsody = null;
	
	public CreateSwimlaneFromParts(IRPApplication rhapsody) {
		this.rhapsody = rhapsody;
	}
	
	public void createSwimlanes() {
		if(rhapsody == null)
			return;
		
		IRPFlowchart activity = null;
		IRPActivityDiagram activityDiagram = null;
		IRPModelElement selectedElem = rhapsody.getSelectedElement();
		if(selectedElem != null && selectedElem instanceof IRPFlowchart) {
			activity = (IRPFlowchart)selectedElem;
			activityDiagram = activity.getFlowchartDiagram();
		}
		else if(selectedElem != null && selectedElem instanceof IRPActivityDiagram) {
			activityDiagram = (IRPActivityDiagram)selectedElem;
			activity = activityDiagram.getFlowchart();
		}
		
		if(activity != null && activityDiagram != null) {
			//check first if swimlanes exists, if so, nothing to do
			IRPCollection swimlanes = activity.getSwimlanes();
			if(swimlanes != null && swimlanes.getCount() > 0)
				return;
			
			//next, check if class owner of the activity has parts, if not, nothing to do
			IRPClassifier classOwner = activity.getItsClass();
			if(classOwner != null) {
				List<IRPInstance> parts = RhpUtils.getAllPartsOfBlock(classOwner);
				if(parts != null && parts.size() > 0) {
					int xPos = 100;
					int yPos = 100;
					int height = 250;
					int width = 150 * parts.size();
					IRPGraphNode frame = activityDiagram.addNewNodeByType("Swimlane Frame", xPos, yPos, width, height);
					activityDiagram.setPropertyValue("Activity_diagram.Swimlane.ShowName", "Represents");
					
					for(int i = 1 ; i <= parts.size() ; i++) {
						xPos = 100 + 150 * i;
						yPos = 150;
						IRPGraphNode graphicalSwimlane = activityDiagram.addNewNodeByType("Swimlane", xPos, yPos, 150, height);
					}
					
					swimlanes = activity.getSwimlanes();
					if(swimlanes != null && swimlanes.getCount() > 0 && parts.size() == swimlanes.getCount()) {
						for(int i = 1 ; i <= swimlanes.getCount() ; i++) {
							IRPSwimlane swimlane = (IRPSwimlane)swimlanes.getItem(i);
							swimlane.setRepresents(parts.get(i - 1));
						}
					}
				}
			}
		}
	}
	
	public static void main(String[] args) {
		IRPApplication rhapsody = RhapsodyAppServer.getActiveRhapsodyApplication();
		CreateSwimlaneFromParts createSwimlaneFromParts = new CreateSwimlaneFromParts(rhapsody);
		createSwimlaneFromParts.createSwimlanes();
	}

}
