package com.ibm.rhapsody.sysML.proxyportWizards;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.Vector;

import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.custom.TreeEditor;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeColumn;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.swt.SWT;

import com.ibm.rhapsody.sysML.proxyportWizards.Utils.DirectionType;
import com.telelogic.rhapsody.core.IRPAttribute;
import com.telelogic.rhapsody.core.IRPClass;
import com.telelogic.rhapsody.core.IRPClassifier;
import com.telelogic.rhapsody.core.IRPCollection;
import com.telelogic.rhapsody.core.IRPDiagram;
import com.telelogic.rhapsody.core.IRPEventReception;
import com.telelogic.rhapsody.core.IRPFlow;
import com.telelogic.rhapsody.core.IRPGraphEdge;
import com.telelogic.rhapsody.core.IRPGraphNode;
import com.telelogic.rhapsody.core.IRPInstance;
import com.telelogic.rhapsody.core.IRPInterfaceItem;
import com.telelogic.rhapsody.core.IRPLink;
import com.telelogic.rhapsody.core.IRPModelElement;
import com.telelogic.rhapsody.core.IRPOperation;
import com.telelogic.rhapsody.core.IRPPackage;
import com.telelogic.rhapsody.core.IRPPort;
import com.telelogic.rhapsody.core.IRPStereotype;
import com.telelogic.rhapsody.core.IRPTag;
import com.telelogic.rhapsody.core.RhapsodyRuntimeException;

public class CreateProxyportComposite extends Composite {
	private IRPClassifier block = null;
	private IRPInstance part = null;
	private IRPPort currentPort = null;
	private IRPPort selectedPort = null;
	private IRPClassifier interfaceBlock = null;
	private List<IRPModelElement> newlyCreatedElementsMain = new LinkedList<IRPModelElement>();
	private List<IRPModelElement> newlyCreatedElementsInner = new LinkedList<IRPModelElement>();
	
	private Combo portNameCombo;
	private Combo interfaceNameCombo;
	private Tree tree;
	private Text multiplicityText;
	
	private List<TreeItem> treeItems = new LinkedList<TreeItem>();
	private Vector<TreeEditor> treeEditors = new Vector<TreeEditor>();
	
	public CreateProxyportComposite(Composite parent, int style, IRPClassifier block, IRPInstance part, IRPPort currentPort) {
		super(parent, style);

		this.block = block;
		this.part = part;
		this.currentPort = currentPort;
		createContents();
		fillTree();
		markExposedPropertiesInTree(true);
	}

	public void refreshTreeFromModel() {
		if(tree != null) {
			Map<String, String> oldData = serializedDataFromTree();
			
			for(TreeEditor te : treeEditors) {
				Control ctrl = te.getEditor();
				if(ctrl != null && ctrl.isDisposed() == false)
					ctrl.dispose();
				te.setItem(null);
			}
			treeEditors.removeAllElements();
			treeItems.clear();
			
			tree.removeAll();
			fillTree();
			markExposedPropertiesInTree(false);
			
			restoreOldData(oldData);
		}
	}

	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}
	
	protected void createContents() {
		if(block == null)
			return;
		
		setLayout(new GridLayout(4, false));
		    
		Label lblName = new Label(this, SWT.NONE);
		lblName.setText("Proxy Port Name");
		    
		portNameCombo = new Combo(this, SWT.BORDER);
		portNameCombo.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));
		portNameCombo.add("");
		List<IRPPort> proxyports = Utils.getProxyportsOfBlock(block);
		if(proxyports != null && proxyports.size() > 0) {
			for(int i = 0 ; i < proxyports.size() ; i++) {
				IRPPort proxyport = (IRPPort)proxyports.get(i);
				portNameCombo.add(proxyport.getName());
				portNameCombo.setData(proxyport.getName(), proxyport);
				if(currentPort != null && currentPort.equals(proxyport)) {
					portNameCombo.select(i+1);
					selectedPort = currentPort;
					interfaceBlock = currentPort.getOtherClass();
				}
			}
		}
		if(selectedPort == null) {
			portNameCombo.setText(getDefaultPortName());
		}
		portNameCombo.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				String portName = portNameCombo.getText();
				if(portName != null) {
					Object data = portNameCombo.getData(portName);
					if(data != null && data instanceof IRPPort) {
						selectedPort = (IRPPort)data;
						IRPClassifier type = selectedPort.getOtherClass();
						if(type != null) {
							interfaceBlock = type;
						}
					}
					else {
						selectedPort = null;
						interfaceBlock = null;
					}
				}
						
				setValuesForMultiplicityAndInterfaceName();
				markExposedPropertiesInTree(true);
			}
		});
		
		Button btnRefresh = new Button(this, SWT.NONE);
		btnRefresh.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				refreshTreeFromModel();
			}
		});
		btnRefresh.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		btnRefresh.setText("Refresh");
		    
		Label lblInterfaceName = new Label(this, SWT.NONE);
		lblInterfaceName.setText("Interface Block Name");
		    
		interfaceNameCombo = new Combo(this, SWT.BORDER);
		interfaceNameCombo.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 3, 1));
		
		interfaceNameCombo.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				String intrBlkName = interfaceNameCombo.getText();
				if(intrBlkName != null) {
					Object data = interfaceNameCombo.getData(intrBlkName);
					if(data != null && data instanceof IRPClassifier) {
						interfaceBlock = (IRPClassifier)data;
					}
					else {
						interfaceBlock = null;
					}
				}
						
				markExposedPropertiesInTree(true);
			}
		});
		
		Label lblMultiplicity = new Label(this, SWT.NONE);
		lblMultiplicity.setText("Multiplicity");
		
		multiplicityText = new Text(this, SWT.BORDER);
		multiplicityText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 3, 1));
		
		setValuesForMultiplicityAndInterfaceName();
		
		tree = new Tree(this, SWT.BORDER | SWT.MULTI | SWT.FULL_SELECTION);
		tree.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 4, 1));
		
		tree.setHeaderVisible(true);
		tree.setLinesVisible(true);
		    
		TreeColumn tblclmnBlockFeatures = new TreeColumn(tree, SWT.NONE);
		tblclmnBlockFeatures.setWidth(125);
		tblclmnBlockFeatures.setText("Block Features");
		
		TreeColumn tblclmnExpose = new TreeColumn(tree, SWT.NONE);
		tblclmnExpose.setWidth(50);
		tblclmnExpose.setText("Expose");
		
		TreeColumn tblclmnDirection = new TreeColumn(tree, SWT.NONE);
		tblclmnDirection.setWidth(77);
		tblclmnDirection.setText("Direction");
		    
		TreeColumn tblclmnInterfaceFeatureName = new TreeColumn(tree, SWT.NONE);
		tblclmnInterfaceFeatureName.setWidth(140);
		tblclmnInterfaceFeatureName.setText("Interface Feature Name");
	}
	
	private void setValuesForMultiplicityAndInterfaceName() {
		if(block == null)
			return;
		
		interfaceNameCombo.removeAll();
		interfaceNameCombo.add("");
		List<IRPClassifier> interfaceBlocks = Utils.getAvailableInterfaceBlocks(block);
		if(interfaceBlocks != null && interfaceBlocks.size() > 0) {
			for(int i = 0 ; i < interfaceBlocks.size() ; i++) {
				IRPClassifier intrBlk = interfaceBlocks.get(i);
				interfaceNameCombo.add(intrBlk.getName());
				interfaceNameCombo.setData(intrBlk.getName(), intrBlk);
				if(interfaceBlock != null && interfaceBlock.equals(intrBlk)) {
					interfaceNameCombo.select(i+1);
				}
			}
		}
		if(interfaceBlock == null) {
			interfaceNameCombo.setText(getDefaultInterfaceName());
		}
		else if(interfaceBlocks == null || interfaceBlocks.contains(interfaceBlock) == false) {
			String name = interfaceBlock.getName();
			interfaceNameCombo.add(name);
			interfaceNameCombo.setData(name, interfaceBlock);
			int idx = interfaceNameCombo.getItemCount();
			interfaceNameCombo.select(idx-1);
		}
		
		if(selectedPort != null) {
			multiplicityText.setText(selectedPort.getMultiplicity());
		}
		else {
			multiplicityText.setText("1");
		}
	}
	
	private String getDefaultPortName() {
		String portName = "";
		if(block != null) {
			portName = "p_" + block.getName();
			String baseName = portName;
			IRPPort port = Utils.findProxyportByName(block, portName);
			for(int i = 1 ; i < 10 && port != null ; i++) {
				portName = baseName + "_" + Integer.toString(i);
				port = Utils.findProxyportByName(block, portName);
			}
		}
		return portName;
	}
	
	private String getDefaultInterfaceName() {
		String interfaceName = "";
		if(block != null) {
			interfaceName = "IB_" + block.getName();
			String baseName = interfaceName;
			IRPPackage pkg = Utils.getPackageOfBlock(block);
			if(pkg != null) {
				IRPClassifier existingInterface = pkg.findClass(interfaceName);
				for(int i = 1 ; i < 10 && existingInterface != null ; i++) {
					interfaceName = baseName + "_" + Integer.toString(i);
					existingInterface = pkg.findClass(interfaceName);
				}
			}
		}
		return interfaceName;
	}
	
	private void fillTree() {
		if(block == null || tree == null)
			return;
		
		IRPCollection attrs = block.getAttributesIncludingBases();
		if(attrs != null && attrs.getCount() > 0) {
			TreeItem parent = Utils.getOrCreateTreeRootItem(tree, "Attributes");
		
			for(int i = 1 ; i <= attrs.getCount() ; i++) {
				final IRPAttribute attr = (IRPAttribute)attrs.getItem(i);
				String attrName = Utils.getNameAndTypeString(attr);
				if(attr.getOwner() != null && (attr.getOwner() instanceof IRPClassifier) && (attr.getOwner().equals(block) == false)) {
					attrName = attr.getOwner().getName() + "." + attrName;
				}
				
				final TreeItem treeItem = Utils.getOrCreateTreeItem(parent, attrName, treeItems);
				treeItem.setData(attr);

				TreeEditor editor = getTreeEditor(treeItem, 1);
				if(editor == null) {
					editor = new TreeEditor(tree);
					Button checkBtn = new Button(tree, SWT.CHECK);
					checkBtn.pack();
					checkBtn.addSelectionListener(new SelectionAdapter() {
						@Override
						public void widgetSelected(SelectionEvent e) {
							Button button = (Button) e.widget;
							TreeEditor editor = getTreeEditor(treeItem, 2);
							if(editor != null && editor.getEditor() instanceof CCombo) {
								CCombo combo = (CCombo)editor.getEditor();
								if(button.getSelection())
									combo.select(1);
								else
									combo.select(0);
							}
				        
							editor = getTreeEditor(treeItem, 3);
							if(editor != null && editor.getEditor() instanceof Text) {
								Text text = (Text)editor.getEditor();
								if(button.getSelection()) {
									//	text.setEditable(true);
									text.setText(attr.getName());
								}
								else{
									//	text.setEditable(false);
									text.setText("");
								}
							}
						}
					});
					editor.minimumWidth = checkBtn.getSize().x;
					editor.horizontalAlignment = SWT.LEFT;
					editor.setEditor(checkBtn, treeItem, 1);
					treeEditors.add(editor);
				
					editor = new TreeEditor(tree);
					CCombo combo = new CCombo(tree, SWT.READ_ONLY);
					combo.add("");
					combo.add("In");
					combo.add("Out");
					combo.add("Required");
					combo.add("Provided");
					editor.grabHorizontal = true;
					editor.setEditor(combo, treeItem, 2);
					treeEditors.add(editor);
			    
					editor = new TreeEditor(tree);
					Text text1 = new Text(tree, SWT.NONE);
					//text1.setText(attr.getName());
					//    text1.setEditable(false);
					editor.grabHorizontal = true;
					editor.setEditor(text1, treeItem, 3);
					treeEditors.add(editor);
				}
			}
			parent.setExpanded(true);
		 }

		IRPCollection opers = block.getInterfaceItemsIncludingBases();
		if(opers != null && opers.getCount() > 0) {
			TreeItem parent = Utils.getOrCreateTreeRootItem(tree, "Operations");
		    
			for(int i = 1 ; i <= opers.getCount() ; i++) {
				IRPInterfaceItem interfaceItem = (IRPInterfaceItem)opers.getItem(i);
				IRPOperation oper = null;
				if(interfaceItem instanceof IRPOperation)
					oper = (IRPOperation)interfaceItem;
				if(oper != null && (oper.getIsCtor() == 1|| oper.getIsDtor() == 1))
					continue;
				String operName = interfaceItem.getName();
				if(interfaceItem.getOwner() != null && (interfaceItem.getOwner() instanceof IRPClassifier) && (interfaceItem.getOwner().equals(block) == false)) {
					operName = interfaceItem.getOwner().getName() + "." + operName;
				}
				
				final TreeItem treeItem = Utils.getOrCreateTreeItem(parent, operName, treeItems);
				treeItem.setData(interfaceItem);
				
				TreeEditor editor = getTreeEditor(treeItem, 1);
				if(editor == null) {
					editor = new TreeEditor(tree);
					Button checkBtn = new Button(tree, SWT.CHECK);
					checkBtn.pack();
					checkBtn.addSelectionListener(new SelectionAdapter() {
						@Override
						public void widgetSelected(SelectionEvent e) {
							Button button = (Button) e.widget;
							TreeEditor editor = getTreeEditor(treeItem, 2);
							if(editor != null && editor.getEditor() instanceof CCombo) {
								CCombo combo = (CCombo)editor.getEditor();
								if(button.getSelection())
									combo.select(1);
								else
									combo.select(0);
							}				  
						}
					});
					editor.minimumWidth = checkBtn.getSize().x;
					editor.horizontalAlignment = SWT.LEFT;
					editor.setEditor(checkBtn, treeItem, 1);
					treeEditors.add(editor);
			    
					editor = new TreeEditor(tree);
					CCombo combo = new CCombo(tree, SWT.READ_ONLY);
					combo.add("");
					combo.add("Required");
					combo.add("Provided");
					editor.grabHorizontal = true;
					editor.setEditor(combo, treeItem, 2);
					treeEditors.add(editor);	      
			    
					editor = new TreeEditor(tree);
					Text text1 = new Text(tree, SWT.READ_ONLY);
					text1.setText(interfaceItem.getName());
					editor.grabHorizontal = true;
					editor.setEditor(text1, treeItem, 3);
					treeEditors.add(editor);
				}
			}		      
			parent.setExpanded(true);
		 }
		
		IRPCollection parts = Utils.getAllPartsWithProxyPorts(block);
		if(parts != null && parts.getCount() > 0) {
			TreeItem parent = Utils.getOrCreateTreeRootItem(tree, "Parts");
		    
		    for(int i = 1 ; i <= parts.getCount() ; i++) {
				IRPInstance part = (IRPInstance)parts.getItem(i);
				String partName = part.getName();
				IRPClassifier type = part.getOtherClass();
				if(type != null) {
					partName = partName + ":" + type.getName();
				}
				TreeItem treeItem = Utils.getOrCreateTreeItem(parent, partName, treeItems);
				treeItem.setData(part);
				
				List<IRPPort> ports = Utils.getProxyportsOfPartType(part);
				if(ports != null && ports.size() > 0) {
					TreeItem innerParent = Utils.getOrCreateTreeItem(treeItem, "Proxyports", null);
					treeItem.setExpanded(true);
					for(int j = 0 ; j < ports.size() ; j++) {
						final IRPPort port = (IRPPort)ports.get(j);
						String portName = port.getName();
						
						final TreeItem portTreeItem = Utils.getOrCreateTreeItem(innerParent, portName, treeItems);
						portTreeItem.setData(port);
						
						TreeEditor editor = getTreeEditor(portTreeItem, 1);
						if(editor == null) {
							editor = new TreeEditor(tree);
							Button checkBtn = new Button(tree, SWT.CHECK);
							checkBtn.pack();
							checkBtn.addSelectionListener(new SelectionAdapter() {
								@Override
								public void widgetSelected(SelectionEvent e) {
									Button button = (Button) e.widget;
									TreeEditor editor = getTreeEditor(portTreeItem, 3);
									if(editor != null && editor.getEditor() instanceof Text) {
										Text text = (Text)editor.getEditor();
										if(button.getSelection()) {
											//		text.setEditable(true);
											text.setText(port.getName());
										}
										else{
											//		text.setEditable(false);
											text.setText("");
										}
									}
								}
							});
							editor.minimumWidth = checkBtn.getSize().x;
							editor.horizontalAlignment = SWT.LEFT;
							editor.setEditor(checkBtn, portTreeItem, 1);
							treeEditors.add(editor);
					    
							editor = new TreeEditor(tree);
							Text text1 = new Text(tree, SWT.NONE);
							//    text1.setEditable(false);
							editor.grabHorizontal = true;
							editor.setEditor(text1, portTreeItem, 3);
							treeEditors.add(editor);
						}
					}
					innerParent.setExpanded(true);
				}
		    }
		    parent.setExpanded(true);
		}
	}
	
	private void markExposedPropertiesInTree(boolean bReset) {
		if(block == null || tree == null)
			return;
		
		for(TreeEditor te : treeEditors) {
			if(te.getEditor() instanceof Button) {
				Button checkBtn = (Button)te.getEditor();
				TreeItem item = te.getItem();
				if(item != null) {
					Object data = item.getData();
					if(data instanceof IRPAttribute) {
						IRPAttribute attr = (IRPAttribute)data;
						IRPAttribute interfaceAttr = (IRPAttribute)Utils.getInterfaceBlockProperty(attr, null, block, interfaceBlock);
						if(interfaceAttr != null) {
							checkBtn.setSelection(true);
						//	checkBtn.setEnabled(false);
						}
						else if(bReset){
							checkBtn.setSelection(false);
						//	checkBtn.setEnabled(true);
						}
													
						TreeEditor comboTE = getTreeEditor(item, 2);
						if(comboTE != null && comboTE.getEditor() instanceof CCombo) {
							CCombo combo = (CCombo)comboTE.getEditor();
							if(interfaceAttr != null) {
								IRPTag tag = interfaceAttr.getTag("direction");
								if(tag != null) {
									String direction = tag.getValue();
									if(direction.equals("In"))
										combo.select(1);
									else if(direction.equals("Out"))
										combo.select(2);
									else if(direction.equals("required"))
										combo.select(3);
									else if(direction.equals("provided"))
										combo.select(4);
								}
							}
							else if(bReset)
								combo.select(0);
						}
						
						TreeEditor textTE = getTreeEditor(item, 3);
						if(textTE != null && textTE.getEditor() instanceof Text) {
							Text text = (Text)textTE.getEditor();
							if(interfaceAttr != null) {
								text.setText(interfaceAttr.getName());
							//	text.setEditable(false);
							}
							else if(bReset){
								text.setText("");
							//	text.setEditable(true);
							}
						}
					}
					else if(data instanceof IRPInterfaceItem) {
						IRPInterfaceItem oper = (IRPInterfaceItem)data;
						IRPInterfaceItem interfaceOper = null;
						if(interfaceBlock != null) {
							interfaceOper = interfaceBlock.findInterfaceItem(oper.getSignature());
						}
						if(interfaceOper != null) {
							checkBtn.setSelection(true);
					//		checkBtn.setEnabled(false);
						}
						else if(bReset) {
							checkBtn.setSelection(false);
					//		checkBtn.setEnabled(true);
						}
						
						TreeEditor comboTE = getTreeEditor(item, 2);
						if(comboTE != null && comboTE.getEditor() instanceof CCombo) {
							CCombo combo = (CCombo)comboTE.getEditor();
							if(interfaceOper != null) {
								IRPTag tag = interfaceOper.getTag("direction");
								if(tag != null) {
									String direction = tag.getValue();
									if(direction.equals("required"))
										combo.select(1);
									else if(direction.equals("provided"))
										combo.select(2);
								}
							}
							else if(bReset)
								combo.select(0);
						}
					}
					else if(data instanceof IRPPort) {
						IRPPort port = (IRPPort)data;
						IRPInstance partContext = Utils.getTreePartContext(item);
						IRPPort interfacePort = (IRPPort) Utils.getInterfaceBlockProperty(port, partContext, block, interfaceBlock);
						if(interfacePort != null) {
							checkBtn.setSelection(true);
						//	checkBtn.setEnabled(false);
						}
						else if(bReset) {
							checkBtn.setSelection(false);
						//	checkBtn.setEnabled(true);
						}
						
						TreeEditor textTE = getTreeEditor(item, 3);
						if(textTE != null && textTE.getEditor() instanceof Text) {
							Text text = (Text)textTE.getEditor();
							if(interfacePort != null) {
								text.setText(interfacePort.getName());
						//		text.setEditable(false);
							}
							else if(bReset) {
								text.setText("");
							//	text.setEditable(true);
							}
						}
					}
				}				
			}
		}
	}
	
	private void restoreOldData(Map<String, String> oldData) {
		if(oldData != null && oldData.size() > 0) {
			for(TreeItem item : treeItems) {
				if(item.getData() != null && item.getData() instanceof IRPModelElement) {
					IRPModelElement elem = (IRPModelElement)item.getData();
					String data = oldData.get(elem.getGUID());
					if(data != null) {
						String expose = null;
						String direction = null;
						String newName = null;
						StringTokenizer st = new StringTokenizer(data, ",");
						if(st.hasMoreTokens())
							expose = st.nextToken();
						if(st.hasMoreElements())
							direction = st.nextToken();
						if(st.hasMoreElements())
							newName = st.nextToken();
						
						if(expose != null) {
							TreeEditor te = getTreeEditor(item, 1);
							if(te != null && te.getEditor() instanceof Button) {
								Button btn = (Button)te.getEditor();
								if(expose.equals("checked"))
									btn.setSelection(true);
								else if(expose.equals("unchecked"))
									btn.setSelection(false);
							}
						}
						
						if(direction != null) {
							TreeEditor te = getTreeEditor(item, 2);
							if(te != null && te.getEditor() instanceof CCombo) {
								CCombo combo = (CCombo)te.getEditor();
								for(int i = 0 ; i < combo.getItemCount() ; i++) {
					   				String comboItem = combo.getItem(i);
					   				if(comboItem != null && comboItem.equals(direction)) {
					   					combo.select(i);
					   					break;
					   				}
					   			}
							}
						}
						
						if(newName != null) {
							TreeEditor te = getTreeEditor(item, 3);
							if(te != null && te.getEditor() instanceof Text) {
								Text text = (Text)te.getEditor();
								text.setText(newName);
							}
						}
					}
				}
			}
		}
	}
	
	private TreeEditor getTreeEditor(TreeItem item, int column) {
		TreeEditor editor = null;
		if(item != null) {
			for(TreeEditor te : treeEditors) {
				if(te.getItem().equals(item) && te.getColumn() == column) {
					editor = te;
					break;
				}
			}
		}
		return editor;
	}
	
	public IRPPort createOrUpdateProxyport() {
		IRPPort proxyPort = null;
		if(block != null) {
			String portName = portNameCombo.getText();
			if(portName == null || portName.isEmpty()) {
				portName = getDefaultPortName();
			}
			String interfaceName = interfaceNameCombo.getText();
			if(interfaceName == null || interfaceName.isEmpty()) {
				interfaceName = getDefaultInterfaceName();
			}
			
			IRPPackage pkg = Utils.getPackageOfBlock(block);
			if(/*shouldCreateInstanceBlock() &&*/ interfaceBlock == null && pkg != null) {
				interfaceBlock = (IRPClass)pkg.addNewAggr("InterfaceBlock", interfaceName);
				newlyCreatedElementsMain.add(interfaceBlock);
			}
	
			proxyPort = selectedPort;
			if(proxyPort == null) {
				proxyPort = (IRPPort)block.addNewAggr("ProxyPort", portName);
				newlyCreatedElementsMain.add(proxyPort);
			}
			proxyPort.setMultiplicity(multiplicityText.getText());
			createProxyportGraphics(proxyPort);
			
			if(interfaceBlock != null && proxyPort != null) {
				populateInterfaceBlockAccordingToTree(interfaceBlock, proxyPort);
				IRPClass oldContract = proxyPort.getContract();
				if(oldContract == null || oldContract.equals(interfaceBlock) == false) {
					proxyPort.setContract((IRPClass)interfaceBlock);
				
					//mark behavioral is needed (i.e. if attributes exists in the interfaceBlock)
					IRPCollection attrs = interfaceBlock.getAttributes();
					if(attrs != null && attrs.getCount() > 0) {
						proxyPort.setIsBehavioral(1);
					}
				}
			}
		}
		return proxyPort;
	}
	
	private void createProxyportGraphics(IRPPort proxyport) {
		if(proxyport != null && block != null) {
			IRPModelElement parent = block;
			if(part != null)
				parent = part;
			IRPDiagram diagram = Utils.getCurrentDiagram();
			IRPGraphNode selectedNode = (IRPGraphNode) Utils.getSelectedGraphicalElement(parent);
			if(selectedNode == null)
				selectedNode = Utils.getGraphicalNode(diagram, parent, null, null);
			if(selectedNode != null && diagram == null)
				diagram = selectedNode.getDiagram();
			if(diagram != null && selectedNode != null) {
				IRPGraphNode portNode = Utils.getGraphicalNode(diagram, proxyport, selectedNode, parent);
				if(portNode == null) {
					Point ptPort = Utils.getCandidatePointForPort(selectedNode);
					if(ptPort != null) {
						diagram.addNewNodeForElement(proxyport, ptPort.x, ptPort.y, 12, 12);
					}
				}
			}
		}
	}
	
/*	private boolean shouldCreateInstanceBlock() {
		boolean bCreateInterface = false;
		TreeItem[] rootItems = tree.getItems();
		if(rootItems != null && rootItems.length > 0) {
			for(TreeItem rootItem : rootItems) {
				if(bCreateInterface)
					break;
				TreeItem[] innerItems = rootItem.getItems();
				if(innerItems != null && innerItems.length > 0) {
					for(TreeItem innerItem : innerItems) {
						Object data = innerItem.getData();
						if(data instanceof IRPAttribute || data instanceof IRPOperation) {
							TreeEditor te = getTreeEditor(innerItem, 1);
							if(te != null && te.getEditor() instanceof Button) {
								Button btn = (Button)te.getEditor();
								if(btn.getSelection() == false)
									continue;
							}
							te = getTreeEditor(innerItem, 2);
							if(te != null && te.getEditor() instanceof CCombo) {
								CCombo directionCombo = (CCombo)te.getEditor();
								String direction = directionCombo.getText();
								if(direction != null && direction.isEmpty() == false) {
									bCreateInterface = true;
									break;
								}
							}
						}
						else if(data instanceof IRPInstance) {
							TreeItem[] partItems = innerItem.getItems();
							if(partItems != null && partItems.length > 0) {
								for(TreeItem partItem : partItems) {
									if(bCreateInterface)
										break;
									if(partItem.getText().equals("Proxyports")) {
										TreeItem[] portItems = partItem.getItems();
										if(portItems != null) {
											for(TreeItem portItem : portItems) {
												Object portItemData = portItem.getData();
												if(portItemData instanceof IRPPort) {
													TreeEditor te = getTreeEditor(portItem, 1);
													if(te != null && te.getEditor() instanceof Button) {
														Button btn = (Button)te.getEditor();
														if(btn.getSelection()) {
															bCreateInterface = true;
															break;
														}
													}	
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}
		return bCreateInterface;
	}*/
	
	private void populateInterfaceBlockAccordingToTree(IRPClassifier interfaceBlock, IRPPort proxyPort) {
		TreeItem[] rootItems = tree.getItems();
		if(rootItems != null && rootItems.length > 0 && interfaceBlock != null) {
			for(TreeItem rootItem : rootItems) {
				TreeItem[] innerItems = rootItem.getItems();
				if(innerItems != null && innerItems.length > 0) {
					for(TreeItem innerItem : innerItems) {
						Object data = innerItem.getData();
						TreeEditor te = getTreeEditor(innerItem, 1);
						if(te != null && te.getEditor() instanceof Button) {
							Button btn = (Button)te.getEditor();
							if(btn.getSelection() == false) {
								if(data instanceof IRPModelElement) {
									IRPModelElement blockProperty = (IRPModelElement)data;
									IRPModelElement interfaceProperty = null;
									if(blockProperty instanceof IRPInterfaceItem) {
										IRPInterfaceItem oper = (IRPInterfaceItem)blockProperty;
										interfaceProperty = interfaceBlock.findInterfaceItem(oper.getSignature());
									}
									else
										interfaceProperty = Utils.getInterfaceBlockProperty(blockProperty, null, block, interfaceBlock);
									if(interfaceProperty != null)
										interfaceProperty.deleteFromProject();
								}
								continue;
							}
						}
						if(data instanceof IRPAttribute) {
							IRPAttribute attr = (IRPAttribute)data;
							String direction = null;
							String newName = null;
							te = getTreeEditor(innerItem, 2);
							if(te != null && te.getEditor() instanceof CCombo) {
								CCombo directionCombo = (CCombo)te.getEditor();
								direction = directionCombo.getText();
							}
							te = getTreeEditor(innerItem, 3);
							if(te != null && te.getEditor() instanceof Text) {
								Text text = (Text)te.getEditor();
								newName = text.getText();
							}
					
							if(direction != null && direction.isEmpty() == false) {
								if(newName == null)
									newName = attr.getName();
								IRPAttribute newAttr = (IRPAttribute) Utils.getInterfaceBlockProperty(attr, null, block, interfaceBlock);
								if(newAttr != null && (newAttr.getName().equals(newName) == false))
									newAttr.setName(newName);
								if(newAttr == null) {
									newAttr = (IRPAttribute) attr.clone(newName, interfaceBlock);
									newlyCreatedElementsInner.add(newAttr);
								}
								if(newAttr != null) {				
									if(direction.equals("In") || direction.equals("Out")) {
										IRPStereotype stereotype = Utils.getStereotype(newAttr, "directedFeature");
										if(stereotype != null)
											newAttr.removeStereotype(stereotype);
										
										stereotype = Utils.getStereotype(newAttr, "flowProperty");
										if(stereotype == null)
											newAttr.changeTo("FlowProperty");
									}
									else if(direction.equals("Provided") || direction.equals("Required")) {
										IRPStereotype stereotype = Utils.getStereotype(newAttr, "flowProperty");
										if(stereotype != null)
											newAttr.changeTo("Attribute");
										
										stereotype = Utils.getStereotype(newAttr, "directedFeature");
										if(stereotype == null)
											newAttr.addStereotype("directedFeature", "Attribute");
										direction = direction.toLowerCase();
									}
									IRPTag tag = newAttr.getTag("direction");
									tag = newAttr.getTag("direction");
									if(tag != null && tag.getValue().equals(direction) == false)
										newAttr.setTagValue(tag, direction);
									
									//add a BindingConnector from the original attr to the new one in the
									//interface
									String flowName = attr.getName() + "_" + newAttr.getName();
									IRPFlow flow = Utils.findFlow(block, attr, newAttr);
									if(flow != null && flow.getName().equals(flowName) == false)
										flow.setName(flowName);
									if(flow == null) {
										flow = block.addFlows(flowName);
										if(flow != null) {
											newlyCreatedElementsInner.add(flow);
											flow.setEnd1(attr);
											flow.setEnd2(newAttr);
											flow.changeTo("BindingConnector");
											tag = flow.getTag("TargetContext");
											if(tag != null && proxyPort != null) {
												flow.setTagElementValue(tag, proxyPort);
											}
										}
									}
								}
							}
						}
						else if(data instanceof IRPInterfaceItem) {
							IRPInterfaceItem oper = (IRPInterfaceItem)data;
							String direction = null;
							te = getTreeEditor(innerItem, 2);
							if(te != null && te.getEditor() instanceof CCombo) {
								CCombo directionCombo = (CCombo)te.getEditor();
								direction = directionCombo.getText();
								if(direction.equals("Provided") || direction.equals("Required")) {
									IRPInterfaceItem newOper = interfaceBlock.findInterfaceItem(oper.getSignature());
									if(newOper == null) {
										newOper = (IRPInterfaceItem)oper.clone(oper.getName(), interfaceBlock);
										newlyCreatedElementsInner.add(newOper);
									}
									if(newOper != null) {
										IRPStereotype stereotype = Utils.getStereotype(newOper, "directedFeature");
										if(stereotype == null) {
											String metaType = "Operation";
											if(oper instanceof IRPEventReception)
												metaType = "Reception";
											if(oper instanceof IRPOperation) {
												IRPOperation operation = (IRPOperation)oper;
												if(operation.getIsTrigger() == 1) {
													metaType = "TriggeredOperation";
												}
											}
											newOper.addStereotype("directedFeature", metaType);
										}
										
										direction = direction.toLowerCase();
										IRPTag tag = newOper.getTag("direction");
										if(tag != null && tag.getValue().equals(direction) == false) {
											newOper.setTagValue(tag, direction);
										}
									}
								}
							}
						}
						else if(data instanceof IRPInstance) {
							IRPInstance part = (IRPInstance)data;
							TreeItem[] partItems = innerItem.getItems();
							if(partItems != null) {
								for(TreeItem partItem : partItems) {
									if(partItem.getText().equals("Proxyports")) {
										TreeItem[] portItems = partItem.getItems();
										if(portItems != null) {
											for(TreeItem portItem : portItems) {
												Object portItemData = portItem.getData();
												if((portItemData instanceof IRPPort) == false)
													continue;
												IRPPort port = (IRPPort)portItemData;
												te = getTreeEditor(portItem, 1);
												if(te != null && te.getEditor() instanceof Button) {
													Button btn = (Button)te.getEditor();
													if(btn.getSelection() == false)
														continue;
												}
												
												String newName = null;
												te = getTreeEditor(portItem, 3);
												if(te != null && te.getEditor() instanceof Text) {
													Text text = (Text)te.getEditor();
													newName = text.getText();
												}
												
												IRPPort newPort = (IRPPort) Utils.getInterfaceBlockProperty(port, part, block, interfaceBlock);
												if(newPort != null && newPort.getName().equals(newName) == false)
													newPort.setName(newName);
												if(newPort == null) {
													newPort = (IRPPort)port.clone(newName, interfaceBlock);
													newlyCreatedElementsInner.add(newPort);
												}
												if(newPort != null) {
													//add a BindingConnector from the original port to the new one in the
													//interface
													String flowName = port.getName() + "_" + newPort.getName();
													IRPFlow flow = Utils.findFlow(block, port, newPort);
													if(flow != null && flow.getName().equals(flowName) == false)
														flow.setName(flowName);
													if(flow == null) {
														flow = block.addFlows(flowName);
														if(flow != null) {
															newlyCreatedElementsInner.add(flow);
															flow.setEnd1ViaPort(part, port);
															flow.setEnd2(newPort);
															flow.changeTo("BindingConnector");
															IRPTag tag = flow.getTag("SourceContext");
															if(tag != null){
																flow.setTagElementValue(tag, part);
															}
															tag = flow.getTag("TargetContext");
															if(tag != null && proxyPort != null) {
																flow.setTagElementValue(tag, proxyPort);
															}
														}
													}
													
													//last, add an inner-connector between the proxyport of the selected port, and the
													//proxyport of the inner part
													createInnerConnectorIfNeeded(proxyPort, part, port);
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}
	}
	
	private void createInnerConnectorIfNeeded(IRPPort fromProxyPort, IRPInstance toInnerPart, IRPPort toInnerPort) {
		if(block != null && fromProxyPort != null && toInnerPart != null && toInnerPort != null) {
			IRPLink connector = null;
			IRPCollection links = block.getLinks();
			if(links != null && links.getCount() > 0) {
				for(int i = 1 ; i <= links.getCount() ; i++) {
					IRPLink link = (IRPLink)links.getItem(i);
					if(link.getTo() != null && link.getTo().equals(toInnerPart) &&
						link.getFrom() != null && link.getFrom().equals(fromProxyPort) &&
						link.getToPort() != null && link.getToPort().equals(toInnerPort)) {
					
						connector = link;
						break;
					}
				}
			}
			if(connector == null) {
				connector = ((IRPClass)block).addLink(fromProxyPort, toInnerPart, null, null, toInnerPort);
				if(connector != null) {
					connector.changeTo("connector");
					newlyCreatedElementsMain.add(connector);
				}
			}
			
			IRPDiagram diagram = Utils.getCurrentDiagram();
			if(connector != null && diagram != null) {
				IRPGraphEdge edge = (IRPGraphEdge) Utils.getGraphicalElement(diagram, connector, null);
				if(edge == null) {
					IRPModelElement modelParentElement = block;
					if(part != null)
						modelParentElement = part;
					IRPGraphNode fromPortNode = Utils.getGraphicalNode(diagram, fromProxyPort, null, modelParentElement);
					IRPGraphNode toPortNode = Utils.getGraphicalNode(diagram, toInnerPort, null, toInnerPart);
					if(fromPortNode != null && toPortNode != null) {
						Point ptNewSrc = Utils.getMiddlePointOfGraphicalNode(fromPortNode, true);
						Point ptNewTrg = Utils.getMiddlePointOfGraphicalNode(toPortNode, true);
						if(ptNewSrc != null && ptNewTrg != null) {
							edge = diagram.addNewEdgeForElement(connector, fromPortNode, ptNewSrc.x, ptNewSrc.y, 
									toPortNode, ptNewTrg.x, ptNewTrg.y);	
							if(edge != null)
								edge.getModelObject();
						}
					}
				}
			}
		}
	}

	public Vector<String> getInterfaceFlowPropertiesAsString(DirectionType direction) {
		Vector<String> attrs = null;
		if(tree != null && block != null && direction != DirectionType.Unspecified) {
			TreeItem[] rootItems = tree.getItems();
			if(rootItems != null && rootItems.length > 0) {
				for(TreeItem rootItem : rootItems) {
					if(rootItem != null && rootItem.getText().equals("Attributes")) {
						TreeItem[] attrItems = rootItem.getItems();
						if(attrItems != null && attrItems.length > 0) {
							for(TreeItem attrItem : attrItems) {
								if(attrItem.getData() instanceof IRPAttribute) {
									IRPAttribute attr = (IRPAttribute)attrItem.getData();
									boolean bAddAttr = false;
									TreeEditor te = getTreeEditor(attrItem, 1);
									if(te != null && te.getEditor() instanceof Button) {
										Button chkBtn = (Button)te.getEditor();
										bAddAttr = chkBtn.getSelection();
									}
											
									if(bAddAttr) {
										boolean bIsFlowProperty = false;
										te = getTreeEditor(attrItem, 2);
										if(te != null && te.getEditor() instanceof CCombo) {
											CCombo combo = (CCombo)te.getEditor();
											String directionStr  = combo.getText();
											if(directionStr != null) {
												if((direction == DirectionType.InOut && (directionStr.equals("In") || directionStr.equals("Out"))) ||
													(direction == DirectionType.In && directionStr.equals("In")) ||
													(direction == DirectionType.Out && directionStr.equals("Out"))) 
												{
													bIsFlowProperty = true;
												}
											}
										}
										
										if(bIsFlowProperty == false)
											continue;
										
										te = getTreeEditor(attrItem, 3);
										if(te != null && te.getEditor() instanceof Text) {
											Text text = (Text)te.getEditor();
											String attrName = text.getText();
											IRPClassifier type = attr.getType();
											if(type != null)
												attrName = attrName + ":" + type.getName();
											
											if(attrs == null)
												attrs = new Vector<String>();
											attrs.add(attrName);
										}
									}
								}
							}
						}
					}
				}
			}
		}
		return attrs;
	}
	
	public Map<String, DirectionType> getInterfaceFlowPropertiesDirectionMap() {
		Map<String, DirectionType> directionMap = null;
		if(tree != null && block != null ) {
			TreeItem[] rootItems = tree.getItems();
			if(rootItems != null && rootItems.length > 0) {
				for(TreeItem rootItem : rootItems) {
					if(rootItem != null && rootItem.getText().equals("Attributes")) {
						TreeItem[] attrItems = rootItem.getItems();
						if(attrItems != null && attrItems.length > 0) {
							for(TreeItem attrItem : attrItems) {
								if(attrItem.getData() instanceof IRPAttribute) {
									IRPAttribute attr = (IRPAttribute)attrItem.getData();
									boolean bAddAttr = false;
									TreeEditor te = getTreeEditor(attrItem, 1);
									if(te != null && te.getEditor() instanceof Button) {
										Button chkBtn = (Button)te.getEditor();
										bAddAttr = chkBtn.getSelection();
									}
											
									if(bAddAttr) {
										boolean bIsFlowProperty = false;
										DirectionType direction = DirectionType.InOut;;
										te = getTreeEditor(attrItem, 2);
										if(te != null && te.getEditor() instanceof CCombo) {
											CCombo combo = (CCombo)te.getEditor();
											String directionStr  = combo.getText();
											if(directionStr != null) {
												if(directionStr.equals("In") || directionStr.equals("Out")) {
													bIsFlowProperty = true;
													if(directionStr.equals("In"))
														direction = DirectionType.In;
													else if(directionStr.equals("Out"))
														direction = DirectionType.Out;
												}
											}
										}
										
										if(bIsFlowProperty == false)
											continue;
										
										te = getTreeEditor(attrItem, 3);
										if(te != null && te.getEditor() instanceof Text) {
											Text text = (Text)te.getEditor();
											String attrName = text.getText();
											IRPClassifier type = attr.getType();
											if(type != null)
												attrName = attrName + ":" + type.getName();
											
											if(directionMap == null)
												directionMap = new HashMap<String, DirectionType>();
											directionMap.put(attrName, direction);
										}
									}
								}
							}
						}
					}
				}
			}
		}
		return directionMap;
	}
	
	public Map<String, IRPAttribute> getInterfaceFlowPropertiesAsMap() {
		Map<String, IRPAttribute> attrMap = null;
		if(tree != null && block != null) {
			TreeItem[] rootItems = tree.getItems();
			if(rootItems != null && rootItems.length > 0) {
				for(TreeItem rootItem : rootItems) {
					if(rootItem != null && rootItem.getText().equals("Attributes")) {
						TreeItem[] attrItems = rootItem.getItems();
						if(attrItems != null && attrItems.length > 0) {
							for(TreeItem attrItem : attrItems) {
								if(attrItem.getData() instanceof IRPAttribute) {
									IRPAttribute attr = (IRPAttribute)attrItem.getData();
									boolean bAddAttr = false;
									TreeEditor te = getTreeEditor(attrItem, 1);
									if(te != null && te.getEditor() instanceof Button) {
										Button chkBtn = (Button)te.getEditor();
										bAddAttr = chkBtn.getSelection();
									}
									
									if(bAddAttr) {
										boolean bIsFlowProperty = false;
										te = getTreeEditor(attrItem, 2);
										if(te != null && te.getEditor() instanceof CCombo) {
											CCombo combo = (CCombo)te.getEditor();
											String direction  = combo.getText();
											if(direction != null && (direction.equals("In") || direction.equals("Out")))
												bIsFlowProperty = true;
										}
										
										if(bIsFlowProperty == false)
											continue;
										
										te = getTreeEditor(attrItem, 3);
										if(te != null && te.getEditor() instanceof Text) {
											Text text = (Text)te.getEditor();
											String attrName = text.getText();
											IRPClassifier type = attr.getType();
											if(type != null)
												attrName = attrName + ":" + type.getName();
											
											if(attrMap == null)
												attrMap = new HashMap<String, IRPAttribute>();
											IRPAttribute interfaceAttr = (IRPAttribute)Utils.getInterfaceBlockProperty(attr, null, block, interfaceBlock);
											attrMap.put(attrName, interfaceAttr);
										}
									}
								}
							}
						}
					}
				}
			}
		}
		return attrMap;
	}
	
	public Map<String, IRPPort> getInterfacePortsAsMap() {
		Map<String, IRPPort> portMap = null;
		for(TreeEditor te : treeEditors) {
			if(te.getEditor() instanceof Button) {
				Button checkBtn = (Button)te.getEditor();
				if(checkBtn.getSelection() == false)
					continue;
				TreeItem item = te.getItem();
				if(item != null && item.getData() instanceof IRPPort) { 
					IRPPort blockPort = (IRPPort)item.getData();
					String interfacePortName = blockPort.getName();
					TreeEditor textTE = getTreeEditor(item, 3);
					if(textTE != null && textTE.getEditor() instanceof Text) {
						Text text = (Text)textTE.getEditor();
						interfacePortName = text.getText();
					}
					
					if(portMap == null) 
						portMap = new HashMap<String, IRPPort>();
					portMap.put(interfacePortName, blockPort);
				}
			}
		}
		return portMap;
	}
	
	public String getInterfaceName() {
		String interfaceName = null;
		if(interfaceNameCombo != null) {
			interfaceName = interfaceNameCombo.getText();
			if(interfaceName != null && interfaceName.isEmpty())
				interfaceName = null;
		}
		return interfaceName;
	}
	
	public String getPortName() {
		String portName = null;
		if(portNameCombo != null) {
			portName = portNameCombo.getText();
		}
		return portName;
	}
	
	public IRPClassifier getInterfaceBlock() {
		return interfaceBlock;
	}
	
	public void rollbackChanges() {
		if(newlyCreatedElementsInner != null && newlyCreatedElementsInner.size() > 0) {
			List<IRPModelElement> flows = new LinkedList<IRPModelElement>();
			List<IRPModelElement> others = new LinkedList<IRPModelElement>();
			for(IRPModelElement elem : newlyCreatedElementsInner) {
				if(elem instanceof IRPFlow)
					flows.add(elem);
				else
					others.add(elem);
			}
			
			for(IRPModelElement flow : flows) {
				flow.deleteFromProject();
			}
			
			for(IRPModelElement other : others) {
				other.deleteFromProject();
			}
			
			flows.clear();
			others.clear();
			newlyCreatedElementsInner.clear();
		}
		
		if(newlyCreatedElementsMain != null && newlyCreatedElementsMain.size() > 0) {
			for(IRPModelElement elem : newlyCreatedElementsMain) {
				if(elem.equals(interfaceBlock))
					interfaceBlock = null;
				elem.deleteFromProject();
			}
			newlyCreatedElementsMain.clear();
		}
	}
	
	private Map<String, String> serializedDataFromTree() {
		Map<String, String> serializedData = null;
		if(treeItems != null && treeItems.size() > 0) {
			serializedData = new HashMap<String, String>();
			for(TreeItem item : treeItems) {
				if(item.getData() != null && item.getData() instanceof IRPModelElement) {
					IRPModelElement elem = (IRPModelElement)item.getData();
					String data = "";
					TreeEditor te = getTreeEditor(item, 1);
					if(te != null && te.getEditor() instanceof Button) {
						Button btn = (Button)te.getEditor();
						if(btn.getSelection())
							data += "checked";
						else
							data += "unchecked";
					}
					
					te = getTreeEditor(item, 2);
					if(te != null && te.getEditor() instanceof CCombo) {
						data += ",";
						CCombo combo = (CCombo)te.getEditor();
						data += combo.getText();
					}
					
					te = getTreeEditor(item, 3);
					if(te != null && te.getEditor() instanceof Text) {
						data += ",";
						Text text = (Text)te.getEditor();
						data += text.getText();
					}
					
					if(data.isEmpty() == false) {
						try {
							String elemGuid = elem.getGUID();
							serializedData.put(elemGuid, data);
						}
						catch (RhapsodyRuntimeException e) {
							//could be elem is deleted therefore 'elem.getGUID' throws an exception
							//in this case don't serialize anything
						}
					}
				}
			}
		}
		return serializedData;
	}
}
