package com.ibm.rhapsody.sysML.proxyportWizards;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Monitor;
import org.eclipse.swt.widgets.Shell;

import com.telelogic.rhapsody.core.IRPApplication;
import com.telelogic.rhapsody.core.IRPClassifier;
import com.telelogic.rhapsody.core.IRPInstance;
import com.telelogic.rhapsody.core.IRPModelElement;
import com.telelogic.rhapsody.core.RhapsodyAppServer;
import com.telelogic.rhapsody.core.RhapsodyRuntimeException;

import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;

public class CreateProxyportShell {
	private IRPApplication rhapsody = null;
	private IRPClassifier block = null;
	private IRPInstance part = null;
	
	private Shell shlCreateProxyport = null;
	CreateProxyportComposite composite = null;
	
	public CreateProxyportShell(IRPApplication rhapsody, IRPClassifier block, IRPInstance part) {
		this.rhapsody = rhapsody;
		this.block = block;
		this.part = part;
	}
	
	public void open() {
		createContents();
		
		Display display = Display.getDefault();
		
		String icoPath = rhapsody.getOMROOT() + "/RhapsodyApps/images/rhapsody.ico";
		Image small = new Image(display, icoPath);
		shlCreateProxyport.setImage(small);
		
		Monitor primary = display.getPrimaryMonitor();
	    Rectangle bounds = primary.getBounds();
	    Rectangle rect = shlCreateProxyport.getBounds();    
	    int x = bounds.x + (bounds.width - rect.width) / 2;
	    int y = bounds.y + (bounds.height - rect.height) / 2;
	    shlCreateProxyport.setLocation(x, y);
	    
	    shlCreateProxyport.open();
	    shlCreateProxyport.layout();
	    shlCreateProxyport.forceActive();
		
		while (!shlCreateProxyport.isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
		}
	}
	
	/**
	 * @wbp.parser.entryPoint
	 */
	protected void createContents() {
		shlCreateProxyport = new Shell(SWT.CLOSE | SWT.TITLE | SWT.BORDER | SWT.APPLICATION_MODAL | SWT.ON_TOP | SWT.RESIZE);
		shlCreateProxyport.setSize(425, 350);
		shlCreateProxyport.setText("Create Proxyport");
		shlCreateProxyport.setLayout(new GridLayout(2, false));
		
		composite = new CreateProxyportComposite(shlCreateProxyport, SWT.NONE, block, part, null);
		composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 2, 1));
		
		Button btnOk = new Button(shlCreateProxyport, SWT.NONE);
		btnOk.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				try {
					composite.createOrUpdateProxyport();
					shlCreateProxyport.dispose();
				}
				catch (RhapsodyRuntimeException ex) {
					MessageBox mb = new MessageBox(shlCreateProxyport, SWT.ERROR | SWT.YES | SWT.NO);
					mb.setText("Error");
					String msg = ex.getMessage();
					msg += "\n";
					msg += "Do you want to rollback changes?";
					mb.setMessage(msg);
					if(mb.open() == SWT.YES) {
						composite.rollbackChanges();
						rhapsody.refreshAllViews();
					}
				}
			}
		});
		btnOk.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, true, false, 1, 1));
		btnOk.setText("OK");
		
		Button btnCancel = new Button(shlCreateProxyport, SWT.NONE);
		btnCancel.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				shlCreateProxyport.dispose();
			}
		});
		btnCancel.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		btnCancel.setText("Cancel");
	}
	
	public static void createDialog(IRPApplication rhapsody) {
		if(rhapsody != null) {
			IRPModelElement elem = rhapsody.getSelectedElement();
			IRPClassifier block = null;
			IRPInstance part = null;
			if(elem instanceof IRPInstance) {
				part = (IRPInstance)elem;
				block = part.getOtherClass();
			}
			if(elem instanceof IRPClassifier) {
				block = (IRPClassifier)elem;
			}
			
			if(block != null) {
				CreateProxyportShell dlg = new CreateProxyportShell(rhapsody, block, part);
				dlg.open();
			}
		}
	}
	
	public static void main(String args[]) {
		createDialog(RhapsodyAppServer.getActiveRhapsodyApplication());
	}
}
