package com.ibm.rhapsody.sysML.proxyportWizards;

import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Monitor;
import org.eclipse.swt.widgets.Shell;

import com.telelogic.rhapsody.core.IRPApplication;
import com.telelogic.rhapsody.core.IRPClassifier;
import com.telelogic.rhapsody.core.IRPInstance;
import com.telelogic.rhapsody.core.IRPLink;
import com.telelogic.rhapsody.core.IRPModelElement;
import com.telelogic.rhapsody.core.IRPPort;
import com.telelogic.rhapsody.core.RhapsodyAppServer;
import com.telelogic.rhapsody.core.RhapsodyRuntimeException;

public class ElaborateConnectorViaPortsWizard extends Wizard {
	private static IRPApplication rhapsody = null;
	private static IRPLink connector = null;
	private static IRPInstance end1Part = null;
	private static IRPInstance end2Part = null;
	private static IRPPort end1Port = null;
	private static IRPPort end2Port = null;
	private static IRPClassifier end1Type = null;
	private static IRPClassifier end2Type = null;
	
	private CreateProxyportPage createProxyportEnd1Page = null;
	private CreateProxyportPage createProxyportEnd2Page = null;
	private MappingProxyportsPage mappingProxyportsPage = null;
	
	public ElaborateConnectorViaPortsWizard() {
		setWindowTitle("Elaborate Connector Via Ports Wizard");	
	}

	private static boolean connectToRhapsody() {
		boolean bConnectOk = false;
		if(rhapsody == null)
			return false;
		IRPModelElement elem = rhapsody.getSelectedElement();
		if(elem instanceof IRPLink) {
			connector = (IRPLink)elem;
			end1Part = connector.getFrom();
			end1Port = connector.getFromPort();
			if(end1Part instanceof IRPPort && end1Port == null) {
				end1Port = (IRPPort)end1Part;
				end1Part = null;
				if(end1Port.getOwner() instanceof IRPClassifier)
					end1Type = (IRPClassifier)end1Port.getOwner();
			}
			if(end1Part != null)
				end1Type = end1Part.getOtherClass();
			
			end2Part = connector.getTo();
			end2Port = connector.getToPort();
			if(end2Part instanceof IRPPort && end2Port == null) {
				end2Port = (IRPPort)end2Part;
				end2Part = null;
				if(end2Port.getOwner() instanceof IRPClassifier)
					end2Type = (IRPClassifier)end2Port.getOwner();
			}
			if(end2Part != null)
				end2Type = end2Part.getOtherClass();
			bConnectOk = true;
		}
		
		return bConnectOk;
	}
	
	private void setupRhapsodyIcon() {
		Shell shell = getShell();
		if(rhapsody != null && shell != null) {
			Display display = shell.getDisplay();
			if(display != null) {
				String icoPath = rhapsody.getOMROOT() + "/RhapsodyApps/images/rhapsody.ico";
				Image small = new Image(display, icoPath);
				shell.setImage(small);
			}
		}
	}
	
	@Override
	public void addPages() {
		connectToRhapsody();
		setupRhapsodyIcon();
		if(end1Type != null) {
			createProxyportEnd1Page = new CreateProxyportPage(rhapsody, end1Type, end1Part, end1Port);
			addPage(createProxyportEnd1Page);
		}
		if(end2Type != null) {
			createProxyportEnd2Page = new CreateProxyportPage(rhapsody ,end2Type, end2Part, end2Port);
			addPage(createProxyportEnd2Page);
		}
		
		if(createProxyportEnd1Page != null && createProxyportEnd2Page != null) {
			mappingProxyportsPage = new MappingProxyportsPage(createProxyportEnd1Page, createProxyportEnd2Page, connector);
			addPage(mappingProxyportsPage);
		}
	}

	@Override
	public IWizardPage getNextPage(IWizardPage page) {
		IWizardPage nextPage = super.getNextPage(page);
		if(nextPage instanceof MappingProxyportsPage) {
			mappingProxyportsPage.updatePageIfNeeded();
		}
		return nextPage;
	}
	 
	@Override
	public boolean performFinish() {
		if(createProxyportEnd1Page != null && createProxyportEnd2Page != null && mappingProxyportsPage != null) {
			Shell shell = getShell();
			Display display = shell.getDisplay();
			Cursor cursor = new Cursor(display, SWT.CURSOR_WAIT);
			shell.setCursor(cursor);
			try {
				
				IRPPort fromPort = createProxyportEnd1Page.createOrUpdateProxyport();
				IRPPort toPort = createProxyportEnd2Page.createOrUpdateProxyport();
				if(fromPort != null && toPort != null) {
					mappingProxyportsPage.createMapping(rhapsody, connector, fromPort, toPort);
				}
			} catch (RhapsodyRuntimeException e) {
				MessageBox mb = new MessageBox(getShell(), SWT.ERROR);
				mb.setText("Error");
				mb.setMessage(e.getMessage());
				mb.open();
			}
			shell.setCursor(null);
			cursor.dispose();
		}
		return true;
	}
	
	public static void openWizard(IRPApplication rhp) {
		rhapsody = rhp;
		if(connectToRhapsody() == false)
			return;
		Shell shell = new Shell(SWT.ON_TOP);
		WizardDialog wizard = new WizardDialog(shell, new ElaborateConnectorViaPortsWizard());

		Display display = shell.getDisplay();
		if(display == null)
			display = Display.getDefault();
		
		Monitor primary = display.getPrimaryMonitor();
	    Rectangle bounds = primary.getBounds();
	    Rectangle rect = shell.getBounds();    
	    int x = bounds.x + (bounds.width - rect.width) / 2;
	    int y = bounds.y + (bounds.height - rect.height) / 2;
	    shell.setLocation(x, y);
	    
		wizard.open();
	}
	
	public static void main(String args[]) {
		rhapsody = RhapsodyAppServer.getActiveRhapsodyApplication();
		openWizard(rhapsody);
	}	
}
